/*
 * NBCT
 * Copyright (c) 2007-2010, Yoshitaka Kameya
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms,  with or without
 * modification,  are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions  in  binary  form  must  reproduce  the  above
 *    copyright notice,  this list of conditions  and  the following
 *    disclaimer  in  the  documentation  and/or   other   materials
 *    provided with the distribution.
 *  * Neither  the  name  of  Tokyo Institute of Technology  nor the
 *    names of its contributors may be used to  endorse  or  promote
 *    products derived from  this  software  without  specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS"  AND  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO,  THE IMPLIED WARRANTIES OF MERCHANTABILITY  AND FITNESS
 * FOR  A  PARTICULAR  PURPOSE  ARE  DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL,    SPECIAL,   EXEMPLARY,   OR   CONSEQUENTIAL   DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT  LIABILITY,  OR  TORT  (INCLUDING  NEGLIGENCE  OR OTHERWISE)
 * ARISING  IN  ANY  WAY  OUT  OF  THE  USE  OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "nbc_def.h"
#include "nbc_ext.h"

#ifndef MAX_LINE_LEN
#define MAX_LINE_LEN (1048576)
#endif

char **alloc_field(int num_field)
{
  int f;
  char **field;

  field = (char **)xmalloc(sizeof(char *)*num_field);
  for (f=0; f<num_field; f++)
    field[f] = alloc_str(MAX_LINE_LEN + 1);

  return field;
}

void release_field(char **field, int num_field)
{
  int f;

  for (f=0; f<num_field; f++)
    xfree(field[f]);
  xfree(field);
}

/*
 * [Note]
 * - split() returns -1 if the number of fields exceeds max_field,
 *   and returns the number of fields found otherwise.
 * - the position of each field will be recorded into pos[].
 *   On the other hand, if pos == NULL, no positions will be recorded.
 */
int split(int max_field, const char *buf, char **field, int *pos,
          const char *delims, const char *skip_chars)
{
  int c,n,f,num_field,start;
  const char *b;
  char *q;

  for (f=0; f<max_field; f++)
    if (pos != NULL) pos[f] = -1;

  n = 0;
  c = 0;
  b = buf;
  while (c < MAX_LINE_LEN && *b != '\0') {
    if (strchr(delims,*b) != NULL) n++;
    b++;
    c++;
  }
  num_field = n + 1;

  if (max_field < num_field) return -1;

  f = 0;
  c = 0;
  b = buf;
  q = field[f];
  start = 1;
  while (c <= MAX_LINE_LEN && *b != '\0') {
    if (strchr(skip_chars,*b)) {
      /* do nothing */
    }
    else if (strchr(delims,*b)) {
      *q = '\0';
      f++;
      q = field[f];
      start = 1;
    }
    else if (*b == '\0' || *b == '\r' || *b == '\n') {
      *q = '\0';
      break;
    }
    else {
      *q = *b;  /* copy */
      if (start && pos != NULL) pos[f] = (b - buf);
      start = 0;
      q++;
    }
    b++;
    c++;
  }
  *q = '\0';

  return num_field;
}

int split_comma(int max_field, const char *buf, char **field, int *pos)
{
  return split(max_field,buf,field,pos,","," \t");
}

int split_tab(int max_field, const char *buf, char **field, int *pos)
{
  return split(max_field,buf,field,pos,"\t"," ");
}
